
//-------------------------------------------------------------
// OpenGL program by Andreas Gustafsson (Shadow/Noice) (C) 2000
//-------------------------------------------------------------
#include "stdafx.h"
#include "Texture.h"

#pragma warning(disable : 4305)
#pragma warning(disable : 4244)

#define NR_STARS 2000
#define NR_ROCKS 30
#define NR_WAVES 24

//----------------------------------------
// Usefull debugger thingy.
void DebugOut(const char *szFormat, ...);
//----------------------------------------

// Change your App Name.
char szAppName[] = "Skeleton";
HWND hwnd;

// Timer info.  Very useful for keeping animated stuff in sync on different computers.
extern float fTime, fDeltaTime;

// Current window dimensions.
extern GLsizei g_w, g_h;

// Use the demonstration texture.
CTexture gTexExample;
CTexture gTexExample2;
int updateFPS=0;
double curFPS=0;
GLUquadricObj *quadObj;
double starx[NR_STARS],stary[NR_STARS],starz[NR_STARS],stari[NR_STARS];
double rockpos[NR_ROCKS];
double rocksx[NR_ROCKS],rocksy[NR_ROCKS],rocksz[NR_ROCKS];
double rockrx[NR_ROCKS],rockry[NR_ROCKS],rockrz[NR_ROCKS];
double rockrax[NR_ROCKS],rockray[NR_ROCKS],rockraz[NR_ROCKS];
double wrot[NR_WAVES],wtime[NR_WAVES],wr[NR_WAVES],wg[NR_WAVES],wb[NR_WAVES],wspd[NR_WAVES],wmax[NR_WAVES];

float random(float high)
{
    return((float)((double)rand()/32768)*high);
}

void initWave(int nr)
{
	wtime[nr]=0;
	wrot[nr]=random(360);
	wr[nr]=random(0.3f)+0.7f;
	wg[nr]=random(0.6f);
	wb[nr]=random(0.3f);
	wspd[nr]=0.5f+random(0.3f);
	wmax[nr]=1.0f+random(0.5f);
}


// Startup Stuff.
bool Init()	// Called right after the window is created, and OpenGL is initialized.
{
	int i;
	float r,v1,v2;

	glViewport (0, 0, (GLsizei) g_w, (GLsizei) g_h); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective(55.0, (GLfloat) g_w/(GLfloat) g_h, 1.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef (0.0, 0.0, -3.0);
	glClearColor(0.0,0.0,0.0,0.0);

	quadObj=gluNewQuadric();
	gluQuadricTexture(quadObj,true); 
	gTexExample.LoadJPG("a104.jpg");
	gTexExample2.LoadJPG("smoke.jpg");

	for(i=0;i<NR_WAVES;i++) 
	{
		initWave(i);
		wtime[i]=3.0f+random(1.0f);
	}


	for(i=0;i<NR_STARS;i++)
	{
		r=5;
		v1=random(2*PI);
		v2=random(2*PI);
		starx[i]=r*cos(v1);
		stary[i]=r*sin(v1)*sin(v2);
		starz[i]=r*sin(v1)*cos(v2);
		stari[i]=random(0.5f)+0.5f;
	}

	for(i=0;i<NR_ROCKS;i++)
	{
		rockpos[i]=random(0.05)-0.025;
		rocksx[i]=0.05*(1+random(2));
		rocksy[i]=0.05*(1+random(2));
		rocksz[i]=0.05*(1+random(2));
		rockrx[i]=random(360);
		rockry[i]=random(360);
		rockrz[i]=random(360);
		rockrax[i]=-1+random(2);
		rockray[i]=-1+random(2);
		rockraz[i]=-1+random(2);
	}
	
	// If everything went well, then return true.
	// Returning false will cause the program to quit right away.
	return true;
}

void drawRock(float px,float py, float pz,float rx,float ry,float rz,float sx,float sy,float sz)
{
	glPushMatrix();
	glTranslatef(px,py,pz);
	glRotatef(rx,1.f,0,0);
	glRotatef(ry,0,1.f,0);
	glRotatef(rz,0,0,1.f);
	glScalef(sx,sy,sz);
	gluSphere(quadObj,0.5f,16,16);
	glPopMatrix();
}

void drawStars()
{
	int i;

	glBegin(GL_POINTS);
	for(i=0;i<NR_STARS;i++)
	{
		glColor3f(stari[i],stari[i],stari[i]);
		glVertex3f(starx[i],stary[i],starz[i]);
	}
	glEnd();
}

void drawWave(int nr)
{
	float colMod=1.0;

	glPushMatrix();
	glScalef(wtime[nr],wtime[nr],wtime[nr]);
	if(wtime[nr]>(wmax[nr]-1.0)) colMod=wmax[nr]-wtime[nr];
//	glRotatef( wrot[nr], 1.f, 0, 0 );
//	glRotatef( wrot[nr], 0, 1.f, 0 );
	glRotatef( wrot[nr], 0, 0, 1.f );
	glColor3f(colMod*wr[nr],colMod*wg[nr],colMod*wb[nr]);
	glBegin( GL_QUADS );
	glTexCoord2f( 0, 0 );
	glVertex3f(-1,-1,0);
	glTexCoord2f( 1, 0 );
	glVertex3f(1,-1,0);
	glTexCoord2f( 1, 1 );
	glVertex3f(1,1,0);
	glTexCoord2f( 0, 1 );
	glVertex3f(-1,1,0);
	glEnd();	
	wtime[nr]+=fDeltaTime*wspd[nr];
	if(wtime[nr]>wmax[nr]) initWave(nr);
	glPopMatrix();
}


void Shutdown() // Called right after the window is destroyed.
{
}


// Draw all the scene related stuff.
void Render()
{
	char s[100];
	int i;
	double r1,r2,r3;

	curFPS+=1.f/fDeltaTime;
	updateFPS++;
	if(updateFPS==20)
	{
		sprintf(s,"Asteroid Belt by Andreas Gustafsson (Shadow/Noice) (C) 2000   -=[ FPS:%.0f ]=-",curFPS/20.0f);
		SetWindowText(hwnd,s);
		updateFPS=0;
		curFPS=0;
	}


	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	r1=fTime*9;
	r2=fTime*12;
	r3=fTime*-6;
	glRotatef(r1, 1.f, 0, 1.f );
	glRotatef(r2, 0, 1.f, 0 );
	glRotatef(r3, 0, 0, 1.f );
	drawStars();
	glEnable( GL_TEXTURE_2D );
	gTexExample.Use();
	glColor3f(1.0f,1.0f,1.0f);
	for(i=0;i<NR_ROCKS;i++)
	{
		drawRock(1.2*cos(0.5*fTime+i*2*PI/NR_ROCKS),
			     1.2*sin(0.5*fTime+i*2*PI/NR_ROCKS),
				 rockpos[i],
				 rockrx[i],
				 rockry[i],
				 rockrz[i],
				 rocksx[i],
				 rocksy[i],
				 rocksz[i]);
		rockrx[i]+=100*fDeltaTime*rockrax[i];		
		rockry[i]+=100*fDeltaTime*rockray[i];		
		rockrz[i]+=100*fDeltaTime*rockraz[i];		
		if(rockrx[i]>360) rockrx[i]-=360;
		if(rockry[i]>360) rockry[i]-=360;
		if(rockrz[i]>360) rockrz[i]-=360;
		if(rockrx[i]<0) rockrx[i]+=360;
		if(rockry[i]<0) rockry[i]+=360;
		if(rockrz[i]<0) rockrz[i]+=360;
	}
	glRotatef(-r3, 0, 0, 1.f );
	glRotatef(-r2, 0, 1.f, 0 );
	glRotatef(-r1, 1.f, 0, 1.f );
	glScalef(0.55,0.55,0.55);
	glBlendFunc(GL_ONE, GL_ONE);
	glEnable(GL_BLEND);
	glDepthMask(0);
	//glDisable(GL_DEPTH_TEST);
	gTexExample2.Use();
	for(i=0;i<NR_WAVES;i++) drawWave(i);
	glDepthMask(1);
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}
